"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var Q = require('q');
var _ = require('lodash');
var access_control_with_organizations_authorization_1 = require("./access-control-with-organizations.authorization");
var controller_utils_1 = require("../utils/controller.utils");
var entity_utils_1 = require("../utils/entity.utils");
var base_query_builder_1 = require("../utils/base.query-builder");
var promise_grants_utils_1 = require("../utils/promise-grants.utils");
var functions_utils_1 = require("../utils/functions.utils");
var AccessControlBaseController = /** @class */ (function () {
    function AccessControlBaseController(model, grants) {
        this.model = model;
        var _resource = model.collection.collectionName;
        this.promisedAc = access_control_with_organizations_authorization_1.crudAccessControlWithOrgRolesFactory(_resource, grants);
        logger_1.default.debug("inited access control for " + _resource);
    }
    AccessControlBaseController.prototype.create = function (req, res, options) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'create', function (grant, user) {
            return Q.when()
                .then(self.entityFromBody(req))
                .then(self.setOwner(grant, user))
                .then(self.applyBeforeUpdateIfDefined(options))
                .then(entity_utils_1.createEntity(self.model));
        }, options);
    };
    AccessControlBaseController.prototype.find = function (req, res, exQueryBuilder, options) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'read', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user, exQueryBuilder))
                .then(base_query_builder_1.createQueryExecutor(self.model))
                .then(base_query_builder_1.execFindAndCount(req.query, res, self.isLean(options)));
        }, options);
    };
    AccessControlBaseController.prototype.findById = function (req, res, exQueryBuilder, options) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'read', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user, exQueryBuilder))
                .then(base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id, options && !!options.transformOut))
                .then(controller_utils_1.handleEntityNotFound(res));
        }, options);
    };
    AccessControlBaseController.prototype.restrictedQueryBuilderFactory = function (grant, user, exQueryBuilder) {
        return function () {
            var restrictedQueryBuilder = function (query) {
                var organizationCodes = user.roles
                    .filter(function (role) { return role.indexOf('$organization') == 0; })
                    .map(function (role) { return role.replace('$organization:', '').replace(/:.*$/, ''); });
                return functions_utils_1.now(query)
                    .then(promise_grants_utils_1.ifGrantedForOwn(grant, base_query_builder_1.restrictByOwner(grant.ownerTypes, user._id, organizationCodes)))
                    .then(promise_grants_utils_1.ifDefined(exQueryBuilder))
                    .value();
            };
            return restrictedQueryBuilder;
        };
    };
    AccessControlBaseController.prototype.update = function (req, res, options) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'update', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user))
                .then(base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id))
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(self.applyBeforeUpdateIfDefined(options))
                .then(entity_utils_1.applyUpdate(req.body))
                .then(self.applyAfterUpdateIfDefined(options));
        }, options);
    };
    AccessControlBaseController.prototype.updateOp = function (req, res, update, options) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'update', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user))
                .then(base_query_builder_1.execfindOneAndUpdateWithQueryBuilder(self.model, req.params.id, update))
                .then(controller_utils_1.handleEntityNotFound(res));
        }, options);
    };
    AccessControlBaseController.prototype.patch = function (req, res, options) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'update', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user))
                .then(base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id))
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(self.applyBeforeUpdateIfDefined(options))
                .then(entity_utils_1.applyPatch(req.body))
                .then(self.applyAfterUpdateIfDefined(options));
        }, options);
    };
    AccessControlBaseController.prototype.remove = function (req, res, options) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'delete', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user))
                .then(base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id))
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(entity_utils_1.removeEntity())
                .then(controller_utils_1.successMessageResult());
        }, options);
    };
    AccessControlBaseController.prototype.setOwner = function (grant, user) {
        return function (entity) {
            logger_1.default.trace('will set owner');
            logger_1.default.trace(grant);
            return functions_utils_1.now(entity)
                .then(promise_grants_utils_1.ifGrantedForUser(grant, entity_utils_1.setUserOwner(user)))
                // .then(ifGrantedForOrganization(grant, setOrganizationOwner(user))) // TODO: will not work anymore. need to be revised!!!
                .value();
        };
    };
    AccessControlBaseController.prototype.isLean = function (options) {
        return options && !!options.transformOut;
    };
    AccessControlBaseController.prototype.applyBeforeUpdateIfDefined = function (options) {
        if (options && !!options.beforeSave) {
            return function (value) { return options.beforeSave(value); };
        }
        else {
            return function (value) { return (value); };
        }
    };
    AccessControlBaseController.prototype.applyAfterUpdateIfDefined = function (options) {
        if (options && !!options.afterSave) {
            return function (value) { return options.afterSave(value); };
        }
        else {
            return function (value) { return (value); };
        }
    };
    AccessControlBaseController.prototype.entityFromBody = function (req) {
        return function () {
            var entity = req.body;
            return entity;
        };
    };
    return AccessControlBaseController;
}());
exports.AccessControlBaseController = AccessControlBaseController;
